"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PassportClient = void 0;
const node_cache_1 = __importDefault(require("node-cache"));
const portal_1 = require("@crysteam/portal");
class PassportClient {
    constructor(options) {
        this._userCache = new node_cache_1.default({ stdTTL: 240 });
        this._portal = options.portal;
        this._host = options.host || "https://passport.crysteam.ru";
        this._customScopes = options.scopes;
        this._initialize();
    }
    _initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            const domain = new URL(this._host).hostname;
            this._availableScopes = this._customScopes || (yield this._portal.getServiceScopesFor(domain));
            this._http = this._portal.createHttpClientFor(this._host, this._availableScopes);
        });
    }
    _hasScope(scope) {
        return !!(this._availableScopes && this._portal.isScopeAllowed(this._availableScopes, scope));
    }
    authorization(data) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._http || !this._hasScope({ method: portal_1.Methods.POST, path: "/auth" })) {
                throw new Error("Insufficient permission to authorization.");
            }
            const { data: tokens } = yield this._http.post("auth", data);
            return tokens;
        });
    }
    refreshToken(refresh_token) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._http || !this._hasScope({ method: portal_1.Methods.POST, path: "/auth/refresh" })) {
                throw new Error("Insufficient permission to authorization.");
            }
            const { data: tokens } = yield this._http.post("auth/refresh", { refresh_token });
            return tokens;
        });
    }
    verify(access_token) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._http || !this._hasScope({ method: portal_1.Methods.GET, path: "/auth" })) {
                throw new Error("Insufficient permission to authorization.");
            }
            const { data: user } = yield this._http.get("auth", { headers: { Authorization: `Bearer ${access_token}` } });
            this._userCache.set(user.id, user);
            return user;
        });
    }
    registerUser(data) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._http || !this._hasScope({ method: portal_1.Methods.POST, path: "/users" })) {
                throw new Error("Insufficient permission to register user.");
            }
            const { data: user } = yield this._http.post("users", data);
            this._userCache.set(user.id, user);
            return user;
        });
    }
    updateUser(id, update) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._http || !this._hasScope({ method: portal_1.Methods.PATCH, path: "/users/:id" })) {
                throw new Error("Insufficient permission to update user.");
            }
            const { data: user } = yield this._http.put(`users/${id}`, update);
            this._userCache.set(user.id, user);
            return user;
        });
    }
    deleteUserByQuery(query) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._http || !this._hasScope({ method: portal_1.Methods.DELETE, path: "/users" })) {
                throw new Error("Insufficient permission to delete user.");
            }
            const { data: user } = yield this._http.delete(`users${query}`);
            this._userCache.del(user.id);
            return user;
        });
    }
    fetchUsers(where_1) {
        return __awaiter(this, arguments, void 0, function* (where, limit = 10) {
            if (!this._http || !this._hasScope({ method: portal_1.Methods.POST, path: "/users/search" })) {
                throw new Error("Insufficient permission to fetch users.");
            }
            const { data } = yield this._http.post(`users/search`, { where, limit });
            return data;
        });
    }
    fetchUserByQuery(query_1) {
        return __awaiter(this, arguments, void 0, function* (query, useCache = true) {
            if (!this._http || !this._hasScope({ method: portal_1.Methods.GET, path: "/users" })) {
                throw new Error("Insufficient permission to fetch user.");
            }
            const { data: user } = yield this._http.get(`users${query}`);
            if (useCache)
                this._userCache.set(user.id, user);
            return user;
        });
    }
    getUserById(id_1) {
        return __awaiter(this, arguments, void 0, function* (id, useCache = true) {
            if (useCache && this._userCache.has(id)) {
                return this._userCache.get(id);
            }
            return this.fetchUserByQuery(`?id=${id}`, useCache);
        });
    }
}
exports.PassportClient = PassportClient;
