import NodeCache from "node-cache";
import { CrysteamPortal, Methods, RoutePermission } from "@crysteam/portal";
import { AxiosInstance } from "axios";

export type UserProfile = {
    id: string;
    username: string;
    locale: string;
    role: string;
    discordId: string;
    minecraftUuid?: string;
    telegramId?: string;
    referralId?: string;
    avatarUri: string;
    lastLoginAt: Date;
    recentIp: string;
};

export type UserRegistrationData = Omit<UserProfile, "id" | "role" | "lastLoginAt" | "recentIp">;

export type PassportOptions = {
    portal: CrysteamPortal;
    scopes?: RoutePermission[];
    host?: string;
};

export type Tokens = {
    accessToken: string;
    refreshToken: string;
}

export class PassportClient {
    private readonly _portal: CrysteamPortal;
    private readonly _host: string;
    private readonly _customScopes?: RoutePermission[];
    private readonly _userCache = new NodeCache({ stdTTL: 240 });

    private _http?: AxiosInstance;
    private _availableScopes?: RoutePermission[];

    constructor(options: PassportOptions) {
        this._portal = options.portal;
        this._host = options.host || "https://passport.crysteam.ru";
        this._customScopes = options.scopes;
        this._initialize();
    }

    private async _initialize(): Promise<void> {
        const domain: string = new URL(this._host).hostname;
        this._availableScopes = this._customScopes || await this._portal.getServiceScopesFor(domain);
        this._http = this._portal.createHttpClientFor(this._host, this._availableScopes as any);
    }

    private _hasScope(scope: RoutePermission): boolean {
        return !!(this._availableScopes && this._portal.isScopeAllowed(this._availableScopes, scope));
    }

    async authorization(data: {code: string, redirectUri: string, referralId?: string}): Promise<Tokens> {
        if (!this._http || !this._hasScope({ method: Methods.POST, path: "/auth" })) {
            throw new Error("Insufficient permission to authorization.");
        }

        const { data: tokens } = await this._http.post("auth", data);
        return tokens;
    }

    async refreshToken(refresh_token: string): Promise<Tokens> {
        if (!this._http || !this._hasScope({ method: Methods.POST, path: "/auth/refresh" })) {
            throw new Error("Insufficient permission to authorization.");
        }

        const { data: tokens } = await this._http.post("auth/refresh", { refresh_token });
        return tokens;
    }

    async verify(access_token: string): Promise<UserProfile> {
        if (!this._http || !this._hasScope({ method: Methods.GET, path: "/auth" })) {
            throw new Error("Insufficient permission to authorization.");
        }

        const { data: user } = await this._http.get("auth", { headers: { Authorization: `Bearer ${access_token}` } });
        this._userCache.set(user.id, user);
        return user;
    }

    async registerUser(data: UserRegistrationData): Promise<UserProfile> {
        if (!this._http || !this._hasScope({ method: Methods.POST, path: "/users" })) {
            throw new Error("Insufficient permission to register user.");
        }

        const { data: user } = await this._http.post("users", data);
        this._userCache.set(user.id, user);
        return user;
    }

    async updateUser(id: string, update: Partial<UserProfile>): Promise<UserProfile> {
        if (!this._http || !this._hasScope({ method: Methods.PATCH, path: "/users/:id" })) {
            throw new Error("Insufficient permission to update user.");
        }

        const { data: user } = await this._http.put(`users/${id}`, update);
        this._userCache.set(user.id, user);
        return user;
    }

    async deleteUserByQuery(query: string): Promise<UserProfile> {
        if (!this._http || !this._hasScope({ method: Methods.DELETE, path: "/users" })) {
            throw new Error("Insufficient permission to delete user.");
        }

        const { data: user } = await this._http.delete(`users${query}`);
        this._userCache.del(user.id);
        return user;
    }

    async fetchUsers(where: object, limit: number = 10): Promise<UserProfile[]> {
        if (!this._http || !this._hasScope({ method: Methods.POST, path: "/users/search" })) {
            throw new Error("Insufficient permission to fetch users.");
        }

        const { data } = await this._http.post(`users/search`, { where, limit }) as { data: UserProfile[] };
        return data;
    }

    async fetchUserByQuery(query: string, useCache = true): Promise<UserProfile> {
        if (!this._http || !this._hasScope({ method: Methods.GET, path: "/users" })) {
            throw new Error("Insufficient permission to fetch user.");
        }

        const { data: user } = await this._http.get(`users${query}`);
        if (useCache) this._userCache.set(user.id, user);
        return user;
    }

    async getUserById(id: string, useCache = true): Promise<UserProfile> {
        if (useCache && this._userCache.has(id)) {
            return this._userCache.get(id) as UserProfile;
        }
        return this.fetchUserByQuery(`?id=${id}`, useCache);
    }
}
